/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;
import java.awt.Image;

public class Bomba extends Elemento {

    private Image bomba; // imagem da bomba
    private Image explosao; // imagem da explosao
    private boolean bombaViva; // indica se a bomba ainda nao explodiu
    private boolean bombaUsada; // verifica se está bomba está sendo utilizada
    private boolean bombaPulsada = true; // faz bomba piscar, se quiser, é possível desenar uma bomba menor
    private boolean explodindo; // indica que a bomba esta' explodindo
    private int tempoDeVidaDaBomba; // indica o tempo que bomba vai demorar para explodir
    private int tempoDeExplosao; // indica o tempo que ira' demorar a explosao
    private int bX; // coordenada X da bomba
    private int bY; // coordenada Y da bomba
    private int bombaL;
    private int bombaC;
    private int posDesenhoX; // deslocamento X do desenho no jPanel
    private int posDesenhoY; // deslocamento Y do desenho no jPanel
    private int forca;

    // método construtor da classe Bomba
    // assim como o a classe Jogador, esta classe Bomba, recebe p do tipo JogoTimer
    // a string com o seu nome, e suas cordenada ( essas não atualizadas a cada vez que são invocadas )
    // 
    public Bomba(JogoTimer p, String imagemNome, int mx, int my) {
        super(p, imagemNome, mx, my);
        this.bomba = this.carregarImagem(imagemNome);
        this.explosao = this.carregarImagem("explosao.png");
        this.bombaViva = false; // verifica se a bomba está viva, assim é desenhada
        this.tempoDeVidaDaBomba = 0; // tempo de vida da bomba
        this.explodindo = false; // verifica se a bomba está explodindo
        this.tempoDeExplosao = 0; // tempo que a bomba fica explodindo
        this.bombaUsada = false; // verifica se a bomba está sendo usada

    }

    @Override
    public void desenhar(Graphics2D g) {
        // enquanto a bomba estiver viva, desenha ela na tela e mantém 
        // o atribudo explodindo em false
        if (bombaViva) {
            if (bombaPulsada) {
                g.drawImage(this.bomba, this.bX, this.bY, this.jogo);
                this.bombaPulsada = false;
            } else {
                this.bombaPulsada = true;
            }
            // depois que o tempo da bomba for menor ou igual a zero, faz a bomba explodir
            if (tempoDeVidaDaBomba <= 0) {
                this.bombaViva = false;
                this.explodindo = true;
                this.bombaUsada = false;
                jogo.cenario[bombaL][bombaC] = 2;
                this.tempoDeExplosao = this.velocidade * 2;
            }
        }
        // depois que a bomba está com explodindo em true, entra nesse if
        if (explodindo) {
            // desenha a bomba na posisão em que foi inserida
            g.drawImage(this.explosao, this.bX, this.bY, this.jogo);//desenha a bomba na própria posição
            // se a bomba encontrar o jogador 1, chama o método desenhaIconeDecrementaVidas(int)
            if (jogo.cenarioJogador[bombaL][bombaC] == 3) {
                desenhaIconeDecrementaVidas(1);
            }
            // se a bomba encontrar o jogador 2, chama o método desenhaIconeDecrementaVidas(int)
            if (jogo.cenarioJogador[bombaL][bombaC] == 4) {
                desenhaIconeDecrementaVidas(2);
            }
            
            // aqui se verifica a explosão para todas as direções
            for (int verificador = 0; verificador < 4; verificador++) { // fica alteradno o valor de X e Y
                for (int i = 1; i <= forca; i++) {
                    int Y = bombaL;
                    int X = bombaC;

                    // ########### lógica para verificar direções ###############
                    if (verificador == 0) { // faz lógica à direita
                        this.posDesenhoX = this.velocidade * i;
                        X = X + i;
                        this.posDesenhoY = 0; // deixa posicao do eixo Y estática ( não varia para cima e baixo )
                    } else if (verificador == 1) {// faz lógica  à esquerda
                        this.posDesenhoX = this.velocidade * i * (-1);
                        X = X + i * (-1);
                        this.posDesenhoY = 0; // deixa posicao do eixo Y estática ( não varia para cima e baixo )
                    } else if (verificador == 2) { // faz lógica  à cima
                        this.posDesenhoY = this.velocidade * i;
                        Y = Y + i;
                        this.posDesenhoX = 0; // deixa posicao do eixo X estática ( não varia para esquerda e direita )
                    } else if (verificador == 3) { // faz lógica à baixo
                        this.posDesenhoY = this.velocidade * i * (-1);
                        Y = Y + i * (-1);
                        this.posDesenhoX = 0; // deixa posicao do eixo X estática ( não varia para esquerda e direita )
                    }
                    // ########### lógica para verificar direções ###############

                    if (jogo.cenario[Y][X] > 2) { // testa se o cenário está dentro do limite
                        if (jogo.cenario[Y][X] > 20) {
                            jogo.cenario[Y][X] = 2;
                            g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                        }
                        if (jogo.cenario[Y][X] == 13) { // verifica se é tijolo com diminui bomba
                            jogo.cenario[Y][X] = 23;
                            g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                            break;
                        }
                        if (jogo.cenario[Y][X] == 14) { // verifica se é tijolo com aumenta bomba
                            jogo.cenario[Y][X] = 24;
                            g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                            break;
                        }
                        if (jogo.cenario[Y][X] == 15) { // verifica se é tijolo com diminui fogo
                            jogo.cenario[Y][X] = 25;
                            g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                            break;
                        }
                        if (jogo.cenario[Y][X] == 16) { // verifica se é tijolo com aumenta fogo
                            jogo.cenario[Y][X] = 26;
                            g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                            break;
                        }
                    } else if (jogo.cenario[Y][X] == 1) { // verifica se é tijolo simples
                        jogo.cenario[Y][X] = 2;
                        g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                        break;
                    } else if (jogo.cenario[Y][X] == 0) { // verifica se é ferro, se verdadeiro, para o for
                        break;
                    } else if (jogo.cenario[Y][X] == 2) { // se for livre, explode e verifica se há jogador

                        g.drawImage(this.explosao, this.bX + this.posDesenhoX, this.bY + this.posDesenhoY, this.jogo);
                        // se a bomba encontrar o jogador 1, chama o método desenhaIconeDecrementaVidas(int)
                        if (jogo.cenarioJogador[Y][X] == 3) {
                            desenhaIconeDecrementaVidas(1);
                        }
                        // se a bomba encontrar o jogador 2, chama o método desenhaIconeDecrementaVidas(int)
                        if (jogo.cenarioJogador[Y][X] == 4) {
                            desenhaIconeDecrementaVidas(2);
                        }
                    }
                }
            }
            // verifica se o tempo de explosão chegou em zero para deixar bomba ser ativada novamente
            if (tempoDeExplosao <= 0) {
                this.explodindo = false;
                this.tempoDeExplosao = this.velocidade * 2;
            }
        }
    }

    @Override
    public void atualizar(int x, int y, int forca) { // Recebe qual jogador está atuando no memento
        
        
        // se a bomba for chamada e não estiver viva, atualiza os valores, e atualiza os atributos
        // altera para viva, e põe a nova cordenada
        if (!this.bombaViva) {
            this.forca = forca;
            this.bombaViva = true;
            this.explodindo = false;
            this.tempoDeExplosao = 0;
            this.tempoDeVidaDaBomba = this.velocidade * 5;
            this.bX = x;
            this.bY = y;
            this.bombaL = y / 40;
            this.bombaC = x / 40;
            jogo.cenario[bombaL][bombaC] = 10;
        } else if (this.bombaViva && this.explodindo) {
            this.bombaViva = false;
        }
        if (bombaViva) {
            this.tempoDeVidaDaBomba -= 10;
        }
        if (explodindo) {
            this.tempoDeExplosao -= 40;
        }
    }

    //public boolean verificaCenario(int X, int Y) {
    //    return (Y > 0 && Y < 17) && (X > 0 && X < 11) == true;
    //}

    public boolean getBombaUsada() {
        return this.bombaUsada == false; // se a bomba estiver sendo usada, retorna true senão retorna false

    }

    public int temJogador(int X, int Y) {
        if (jogo.cenarioJogador[Y][X] == 3) { //  se jogo.cenarioJogador = 3, então tem jogador 1 na posição
            return 1;
        } else if (jogo.cenarioJogador[Y][X] == 4) {  //  se jogo.cenarioJogador = 4, então tem jogador 2 na posição
            return 2;
        }
        return 0;

    }

    public void setBombaUsada() {
        this.bombaUsada = true; // seta bomba como true, para indicar que está sendo utilizada
    }

    // verifica qual jogador foi passado, retira uma vida do painel principal ( ícone )
    // e retira uma vida do jogador específico
    public void desenhaIconeDecrementaVidas(int jogador) {
        if (jogador == 1) {
            if (jogo.jog1.vidas == 1) {
                jogo.framePrincipal.jLabelVida3P1.setIcon(null);
                jogo.jog1.vidas--;
            } else if (jogo.jog1.vidas == 2) {
                jogo.framePrincipal.jLabelVida2P1.setIcon(null);
                jogo.jog1.vidas--;
            } else if (jogo.jog1.vidas == 3) {
                jogo.framePrincipal.jLabelVida1P1.setIcon(null);
                jogo.jog1.vidas--;
            }
            jogo.jog1.jogadorPulsando = false;
            jogo.jog1.tempoPiscando = 5;

        } else {

            if (jogo.jog2.vidas == 1) {
                jogo.framePrincipal.jLabelVida3P2.setIcon(null);
                jogo.jog2.vidas--;
            } else if (jogo.jog2.vidas == 2) {
                jogo.framePrincipal.jLabelVida2P2.setIcon(null);
                jogo.jog2.vidas--;
            } else if (jogo.jog2.vidas == 3) {
                jogo.framePrincipal.jLabelVida1P2.setIcon(null);
                jogo.jog2.vidas--;
            }
            jogo.jog2.jogadorPulsando = false;
            jogo.jog2.tempoPiscando = 5;
        }
    }
}
