/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;

/**
 *
 * @author Emerson Ribeiro de Mello
 */
public class Jogador extends Elemento{

    private Bomba bombaJogador[];
    protected int cima; // código da tecla para mover para cima
    protected int baixo;// código da tecla para mover para baixo
    protected int direita;// código da tecla para mover para direita
    protected int esquerda;// código da tecla para mover para esquerda
    protected int atirar;// código da tecla para atirar
    private int numeroDeBombas;
    private int forcaBomba;
    public int vidas;
    public boolean jogadorPulsando;
    public int tempoPiscando;
    private short souOJogador;


    /* poderia melhorar isso aqui */
    /**
     * 
     * @param pai referencia a area de Desenho
     * @param imagemNome - Nome da imagem que sera' usada para representar o jogador
     * @param bombaNome  - Nome da imagem que sera' usada para representar a bomba deste jogador
     * @param mX Linha na matriz
     * @param mY Coluna na matriz
     * @param c codigo da tecla que sera' usado para mover para cima
     * @param b codigo da tecla que sera' usado para mover para baixo
     * @param d codigo da tecla que sera' usado para mover para esquerda
     * @param e codigo da tecla que sera' usado para mover para direita
     * @param a codigo da tecla que sera' usado para atirar
     */
    // construtor do jogador
    public Jogador(JogoTimer pai, String imagemNome, String bombaNome,
            int mx, int my, int c, int b, int d, int e, int a, short souQuem) {
        super(pai, imagemNome, mx, my);
        this.icone = this.carregarImagem(this.imagemNome);
        this.cima = c;
        this.baixo = b;
        this.direita = d;
        this.esquerda = e;
        this.atirar = a;
        this.numeroDeBombas = 1;
        this.souOJogador = souQuem;
        this.forcaBomba = 1;
        this.vidas = 3;
        this.jogadorPulsando = true;
        this.tempoPiscando = 5;
           // cria 5 bombas para cada jogador, no jogo, o jogador não pode ter mais que 5 bombas
        // por isso não usei o arraylist, com mais tempo, vou fazer
        this.bombaJogador = new Bomba[5];
        for (int i = 0; i < 5; i++) {
            this.bombaJogador[i] = new Bomba(this.jogo, "bomba.png", mx, my);
        }
        /*************************************************/
    }

    /**
     * Desenha a imagem no contexto gráfico do Pai
     * @param g 
     */
    @Override
    public void desenhar(Graphics2D g) {
        for (int i = 0; i < 5; i++) {
            if (!this.bombaJogador[i].getBombaUsada()) {
                this.bombaJogador[i].desenhar(g);
            }
        }
        /*************************************************/
        // desenhando o icone do Jogador na Tela
        if (this.jogadorPulsando || this.tempoPiscando == 0) {
            g.drawImage(this.icone, this.posX, this.posY, this.jogo);
            if (this.tempoPiscando <= 0) {// para dar o efeito do jogador piscando no início e quando morre
            } else {
                this.jogadorPulsando = false;
            }
            this.tempoPiscando--;
        } else {
            this.jogadorPulsando = true;
            this.tempoPiscando--;
        }

    }

    /**
     * Atualiza coordenadas, posicao na matriz e a bomba
     */
    @Override
    public void atualizar(int nulo, int nulo2, int nulo3) {

        int destinoLin = matLinha; // linha destino apos o movimento
        int destinoCol = matColuna; // coluna destino apos o movimento
        int destinoX = posX; // coordenada X destino na area de desenho
        int destinoY = posY; // coordenada Y destino na area de desenho



        if (jogo.framePrincipal.teclado.teclaApertada(this.cima)) {
            destinoY = posY - this.velocidade;
            destinoLin--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.baixo)) {
            destinoY = posY + this.velocidade;
            destinoLin++;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.esquerda)) {
            destinoX = posX - this.velocidade;
            destinoCol--;
        } else if (jogo.framePrincipal.teclado.teclaApertada(this.direita)) {
            destinoX = posX + this.velocidade;
            destinoCol++;
        }

        // se estiver a celula destino estiver livre, mova o personagem para la'
        // nos métodos abaixo, já verifica se há itens para pegar
        if (jogo.estaLivre(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
        } else if (jogo.temBombaMenos(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
            jogo.cenario[matLinha][matColuna] = 2;
            if (numeroDeBombas > 1) {
                numeroDeBombas--;
            }
        } else if (jogo.temBombaMais(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
            jogo.cenario[matLinha][matColuna] = 2;
            numeroDeBombas++;
        } else if (jogo.temFogoMenos(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
            jogo.cenario[matLinha][matColuna] = 2;
            if (forcaBomba > 1) {
                forcaBomba--;
            }
        } else if (jogo.temFogoMais(destinoLin, destinoCol)) {
            matLinha = destinoLin;
            matColuna = destinoCol;
            posX = destinoX;
            posY = destinoY;
            jogo.cenario[matLinha][matColuna] = 2;
            forcaBomba++;
        }

        jogo.cenarioJogador[matLinha][matColuna] = this.souOJogador; // atualiza no cenário, que jogador ele é
        cenarioJogador(souOJogador);
        // quando pressionado a tecla atirar, testa se há bomba disponível
        if (jogo.framePrincipal.teclado.teclaApertadaPrimeiraVez(this.atirar)) {
            for (int i = 0; i < this.numeroDeBombas; i++) { // verifica todas as bombas que o jogador possui
                if (this.bombaJogador[i].getBombaUsada()) { // verificando se há bomba disponível
                    this.bombaJogador[i].setBombaUsada();
                    i = 6;
                }
            }
        }
        // atualiza a bomba, se ela for utilizada
        for (int i = 0; i < 5; i++) {
            if (!this.bombaJogador[i].getBombaUsada()) {
                this.bombaJogador[i].atualizar(this.posX, this.posY, this.forcaBomba);
            }
        }
    }
    // atualiza a matriz do jogador, deixa todos os espaçoes livres ( da matriz do jogador )
    // e põe o valor do jogador, na matriz dele, se for o jogador 1 põe valor 3, se for jogador 2 põe valor 4
    public void cenarioJogador(short jogador) {
        for (int C = 0; C < 17; C++) {
            for (int L = 0; L < 11; L++) {
                if (jogo.cenarioJogador[L][C] == jogador) { // verifica qual jogador está atuando
                    jogo.cenarioJogador[L][C] = 2;
                }
            }
        }
        // depois que todos os valores foram pra 2, insere o valor do jogador corrente
        jogo.cenarioJogador[matLinha][matColuna] = this.souOJogador; 
    }
 // os métodos abaixo, servem para alterar as teclas com o jogo rodando
    public void setAtirar(int atirar) {
        this.atirar = atirar;
    }

    public void setBaixo(int baixo) {
        this.baixo = baixo;
    }

    public void setCima(int cima) {
        this.cima = cima;
    }

    public void setDireita(int direita) {
        this.direita = direita;
    }

    public void setEsquerda(int esquerda) {
        this.esquerda = esquerda;
    }
}
