/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 * 
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import principal.JavaBomber;
import java.awt.event.ActionEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.util.Random;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.Timer;

/**
 * Essa clase contém a lógica principal do Jogo. A atualização da tela é feita
 * através de o Timer do pacote Swing. Por este motivo, essa classe tem que
 * implementar a Interface ActionListener
 * Essa classe herda de Canvas, pois a classe Canvas é ideal para fazer desenhos
 * e permite usufrir o bufferStrategy para melhorar a taxa de atualização da tela
 * 
 * @author Emerson Ribeiro de Mello
 * 
 * CLASSE JOGO, É A CLASSE QUE CRIA O JOGO.
 */
public class JogoTimer extends Canvas implements ActionListener {

    public JavaBomber framePrincipal; // frame principal do jogo
    public Jogador jog1; // Jogador1
    public Jogador jog2; // Jogador2
    private Timer timer; // Timer responsável pela atualização do jogo
    public int larguraBloco; // Dimensao dos personabens, fundos. É 40
    public int colunas; // total de colunas na matriz do jogo
    public int linhas; // total de linhas na matriz do jogo
    private Image imgcenarios[]; // contém as imagens de fundo do cenário
    private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
    private short jogador1 = 3;
    private short jogador2 = 4;
    private int cima; // código da tecla para mover para cima
    private int baixo;// código da tecla para mover para baixo
    private int direita;// código da tecla para mover para direita
    private int esquerda;// código da tecla para mover para esquerda
    private int atirar;// código da tecla para atirar
    private int vitoriasP1;
    private int vitoriasP2;

// CONSTRUTOR DA CALSSE
    public JogoTimer(JavaBomber p) {
        super();
        this.framePrincipal = p;
        this.setBackground(Color.GRAY);
        this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
        this.larguraBloco = 40;
        this.colunas = 17;
        this.linhas = 11;
        this.taxaDeAtualizacao = 120;
        this.imgcenarios = new Image[31];
        this.imgcenarios[0] = (new ImageIcon(this.getClass().getResource("/imagens/ferro.png"))).getImage();
        this.imgcenarios[1] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[13] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[23] = (new ImageIcon(this.getClass().getResource("/imagens/bombamenos.png"))).getImage();
        this.imgcenarios[14] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[24] = (new ImageIcon(this.getClass().getResource("/imagens/bombamais.png"))).getImage();
        this.imgcenarios[15] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[25] = (new ImageIcon(this.getClass().getResource("/imagens/menosfogo.png"))).getImage();
        this.imgcenarios[16] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();
        this.imgcenarios[26] = (new ImageIcon(this.getClass().getResource("/imagens/maisfogo.png"))).getImage();


    }
    // CÓDIGOS QUE REFERENCIA OS DESENHOS NA MATRIZ
    // 0 = ferro
    // 1 = tijolo
    // 2 = vazio
    // 3 = Player 1
    // 4 = Player 2
    // 10 = bomba no cenário
    // 13 = tijolo com diminuição de bombas
    // 23 = diminui bombas
    // 14 = tijolo com aumento de bombas
    // 24 = diminui bombas
    // 15 = tijolo com diminuição de fogo
    // 25 = diminuição de fogo
    // 16 = tijolo com aumento de fogo
    // 26 = aumento de fogo
    // 
    // 
    //... Espaços para mais itens, como add vida etc
    // MATRIZ DO JOGO, ONDE ESTÃO OS ELEMENTOS
    public short[][] cenario = { //11x17 (linha,coluna)
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    };
    // MATRIZ PARA INSERIR OS JOGADORES, DIRERENCIAR UM JOGADOR DO OUTRO
    public short[][] cenarioJogador = { //11x17 (linha,coluna)
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0, 2, 0},
        {0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
        {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
    };

    /**
     * Método invocado para iniciar a partida
     */
    public void comecaOJogo() {
        // criando o Timer com taxa de atualização de N milisegundos
        this.timer = new Timer(taxaDeAtualizacao, this);
        this.framePrincipal.jogoEncerrado = false;
        // criando 2 buffer para acelerar a taxa de desenho
        createBufferStrategy(2);
        randomCenario();

        // criando Jogador 1
        //
        // método retorna. Tecla exemplo this.framePrincipal.retornarTecla(1,5)
        // Se jogador = 1, altera teclas do jogador 1. Se jogador = 2, altera teclas do jogador 2
        // 1 cima, 2 baixo, 3 direita, 4 esquerda, 5 atelra disparo
        // construtor jogador -> JogoTimer pai, String imagemNome, String bombaNome,
        //    int mx, int my, int c, int b, int d, int e, int a, JLabel jLx, JLabel jLy, short souQuem
        
        this.jog1 = new Jogador(this, "azul.png", "bomba.png", 1, 1,
                this.framePrincipal.retornarTecla(1, 1), this.framePrincipal.retornarTecla(1, 2),
                this.framePrincipal.retornarTecla(1, 3), this.framePrincipal.retornarTecla(1, 4),
                this.framePrincipal.retornarTecla(1, 5), jogador1);

        // criando Jogador 2
        //
        // retornarTecla. Tecla exemplo this.framePrincipal.retornarTecla(2,5)
        // Se jogador = 1, altera teclas do jogador 1. Se jogador = 2, altera teclas do jogador 2
        // 1 cima, 2 baixo, 3 direita, 4 esquerda, 5 atelra disparo
        // construtor jogador -> JogoTimer pai, String imagemNome, String bombaNome,
        //    int mx, int my, int c, int b, int d, int e, int a, JLabel jLx, JLabel jLy, short souQuem

        this.jog2 = new Jogador(this, "vermelho.png", "bomba.png", 15, 9,
                this.framePrincipal.retornarTecla(2, 1), this.framePrincipal.retornarTecla(2, 2),
                this.framePrincipal.retornarTecla(2, 3), this.framePrincipal.retornarTecla(2, 4),
                this.framePrincipal.retornarTecla(2, 5), jogador2);

        // Disparando o timer para começar a atualização do Loop principal do Jogo
        // Todo o conteúdo do método actionPerformed dessa classe será executado
        // a cada 60 milisegundos.
        this.timer.start();
    }
    // pausa e despausa o jogo
    public void pausarOJogo(boolean pause) {
        if (pause == false) {
            this.timer.stop();
        } else {
            this.timer.start();
        }
    }
    // reinicia o jogo, reorganiza a matriz, e redistribui os itens
    public void reiniciaOJogo() {
        finalizar();
        for (int l = 0; l < 11; l++) {
            for (int c = 0; c < 17; c++) {
                if (l == 0 || l == 10 || c == 0 || c == 16 || (l % 2 == 0 && c % 2 == 0)) {
                    this.cenario[l][c] = 0;
                    this.cenarioJogador[l][c] = 0;
                } else {
                    this.cenario[l][c] = 2;
                    this.cenarioJogador[l][c] = 2;
                }
            }
        }
        // insere as vidas dos jogadores no frame principal
        this.framePrincipal.jLabelVida1P1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/azul.png")));
        this.framePrincipal.jLabelVida2P1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/azul.png")));
        this.framePrincipal.jLabelVida3P1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/azul.png")));
        this.framePrincipal.jLabelVida1P2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/vermelho.png")));
        this.framePrincipal.jLabelVida2P2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/vermelho.png")));
        this.framePrincipal.jLabelVida3P2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/vermelho.png")));
    }

    /**
     * Principal METODO do JOGO, pois e' aqui que fica a logica que fara':
     * - processamento da logica do jogo (morreu, disparou, etc..)
     * - atualizar as coordenadas dos elementos na tela para o proximo redesenho
     * - redesenhar os elementos na tela
     * 
     * Este método é invocado a cada N milisegundos pelo Timer
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        this.framePrincipal.teclado.poll(); // processa os eventos no teclado
        this.processarLogica(); // atualiza as coordenadas dos elementos
        this.renderizar(); // desenha os elementos no buffer de desenho
        this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao

    }

    /**
     * Atualiza as coordenadas/estados de todos os elementos presentes no jogo
     * No caso, somente 2 jogadores, mas aqui pode-se colocar qualquer outro
     * elemento que venha a ser incluido no jogo
     */
    public void processarLogica() {
        //Chama o atualizar dos elementos
        // verifica se ambos os jogadores chegaram a zero vidas
        if (this.jog1.vidas == 0 && this.jog2.vidas == 0) {
            JOptionPane.showMessageDialog(null, "Empate");
            finalizar();
            this.framePrincipal.jButtonIniciar.setEnabled(false);
            // verifica se jogador 1 perdeu
        } else if (this.jog1.vidas == 0) {
            this.vitoriasP2++; // se p1 pedeu, aumenta quantidade de vitórias de p2
            this.framePrincipal.jLabelVitoriasP2.setText("Vitórias: " + this.vitoriasP2);
            JOptionPane.showMessageDialog(null, "Jogador 2 é o vencedor!");
            finalizar();
            this.framePrincipal.jButtonIniciar.setEnabled(false);
            // verifica se jogador 2 perdeu
        } else if (this.jog2.vidas == 0) {
            this.vitoriasP1++; // se p2 pedeu, aumenta quantidade de vitórias de p1
            this.framePrincipal.jLabelVitoriasP1.setText("Vitórias: " + this.vitoriasP1);
            JOptionPane.showMessageDialog(null, "Jogador 1 é o vencedor!");
            finalizar();
            this.framePrincipal.jButtonIniciar.setEnabled(false);
        }

        if (this.jog1 != null) {
            this.jog1.atualizar(0, 0, 0); // as coordenadas só servem para a bomba, já que não usei o arraylist
        }

        if (this.jog2 != null) {
            this.jog2.atualizar(0, 0, 0); // as coordenadas só servem para a bomba, já que não usei o arraylist
        }
    }

    /**
     * Renderiza no buffer secundário todos os elementos já com suas coordenadas/estados
     * atualizados.
     */
    public void renderizar() {
        Graphics g = getBufferStrategy().getDrawGraphics();
        Graphics2D g2 = (Graphics2D) g;
        //Limpando a tela

        // Gerando um gradiente que ficara' como cor de fundo
        //GradientPaint gp = new GradientPaint(25, 25, Color.white,50, 50, Color.RED, true);
        //g2.setPaint(gp);

        // ou pintando com uma cor definida por RGB
        Color c = new Color(255, 210, 174);
        g2.setColor(c);
        g2.fillRect(0, 0, getWidth(), getHeight());

        // desenhar cenario de fundo
        this.renderizarCenario(g2);

        // desenhando os elementos nao nulos
        if (this.jog1 != null) {
            this.jog1.desenhar(g2);
        }
        if (this.jog2 != null) {
            this.jog2.desenhar(g2);
        }

        //liberando os contextos gráficos
        g.dispose();
        g2.dispose();

    }

    /**
     * Efetuando a troca das areas de desenho e apresentacao. 
     * No metodo redenrizar os elemtos graficos foram desenhados em um buffer
     * secundario, agora iremos fazer com que esse buffer secundario torne-se
     * primario, ou seja, que apareca de fato na tela
     */
    public void desenhar() {
        if (!getBufferStrategy().contentsLost()) {
            getBufferStrategy().show();
        }
    }

    /**
     * Fim de jogo/rodada
     */
    public void finalizar() {
        this.jog1 = null;
        this.jog2 = null;
        this.framePrincipal.jogoEncerrado = true; // utilizado para não chamar os objetos do jogo
        this.timer.stop();
    }

    /**
     * Desenha o cenário com as paredes de tijolos e de ferro
     * @param g contexto gráfico
     */
    public void renderizarCenario(Graphics2D g) {
        for (int i = 0; i < 11; i++) { // linhas
            for (int j = 0; j < 17; j++) { // colunas
                if (this.cenario[i][j] != 2) { // se não for vazio, desenhe
                    g.drawImage(this.imgcenarios[this.cenario[i][j]], this.larguraBloco * j, this.larguraBloco * i, this.framePrincipal);
                }
            }
        }
    }

    /**
     * Verifica se na posicao lin,col da matriz tem um espaço vazio
     * @param lin numero da linha
     * @param col numero da coluna
     * @return true se estiver vazio, false caso contrario
     */
    // verifica se a matriz encontrou espaço vazio, jogador 1 ou jogador 2
    public boolean estaLivre(int lin, int col) {
        return cenario[lin][col] == 2 || cenario[lin][col] == 3 || cenario[lin][col] == 4; // se for == 2, entao ira' retornar true, false caso contrario
    }
    // verifica se encontrou item menos bomba
    public boolean temBombaMenos(int lin, int col) {
        return cenario[lin][col] == 23; // se for == 3, entao ira' retornar true, false caso contrario
    }

    // verifica se encontrou item mais bomba
    public boolean temBombaMais(int lin, int col) {
        return cenario[lin][col] == 24; // se for == 4, entao ira' retornar true, false caso contrario
    }

    // verifica se encontrou item menos fogo
    public boolean temFogoMenos(int lin, int col) {
        return cenario[lin][col] == 25; // se for == 5, entao ira' retornar true, false caso contrario
    }

    // verifica se encontrou item mais fogo
    public boolean temFogoMais(int lin, int col) {
        return cenario[lin][col] == 26; // se for == 6, entao ira' retornar true, false caso contrario
    }

    /**
     * Indica em que posicao na tela encontrase o inicio dessa o retangulo que 
     * compreende uma celula
     * @param celula posicao na matriz que deseja-se descobrir
     * @return Coordenada na tela
     */
    public int celulaParaPixel(int celula) {
        return larguraBloco * celula;

    }

    /**
     * Indica qual celula o pixel esta' contido na matriz
     * @param pixel
     * @return 
     */
    public int pixelParaCelula(int pixel) {
        return pixel / larguraBloco;
    }

    public void randomCenario() {
        //101 que podem ser tijolos
        // posições que não devem ser alteradas:
        // P1 [1][1],[1][2],[2,1]
        // P2 [9][15],[9][14],[8,15]
        // Ferro [par][par] - L%2 == 0 && C%2 == 0 ( todo ferro tem uma linha e uma coluna par)

        Random l = new Random(); // gera um random para linha
        Random c = new Random(); // gera um random para coluna
        Random q = new Random(); // quantidade de itens
        Random quantidadeTijolo = new Random();
        int L = 0;
        int C = 0;
        int Q = 0;
        for (int i = 0; i < quantidadeTijolo.nextInt(10) + 80; i++) { // Sorteia entre 80 e 90 % de tijolos
            while (true) {
                L = l.nextInt(9) + 1;
                C = c.nextInt(15) + 1;
                if (!((L == 1 && C == 1) // se for casa diferentes de ferro ou casa onde nasce o jogador, ponha tijolos
                        || (L == 1 && C == 2)
                        || (L == 2 && C == 1)
                        || (L == 9 && C == 15)
                        || (L == 9 && C == 14)
                        || (L == 8 && C == 15)
                        || (L % 2 == 0 && C % 2 == 0))) {
                    if (this.cenario[L][C] != 1) { // evita sobrescrever tijolos
                        this.cenario[L][C] = 1; // marca tijolo na matriz
                        break;
                    }
                }
            }
        }

        // disposição do icone diminui bomba
        Q = q.nextInt(4) + 1; // máximo 4 itens
        for (int i = 0; i < Q; i++) {
            L = l.nextInt(9) + 1;
            C = c.nextInt(15) + 1;
            if (cenario[L][C] == 1) { // verifica se é tijolo
                cenario[L][C] = 13;
            } else {
                i--;
            }
        }
        // disposição do icone aumenta bomba
        Q = q.nextInt(4) + 1; // máximo 4 itens
        for (int i = 0; i < Q; i++) {
            L = l.nextInt(9) + 1;
            C = c.nextInt(15) + 1;
            if (cenario[L][C] == 1) { // verifica se é tijolo
                cenario[L][C] = 14;
            } else {
                i--;
            }
        }
        // disposição do icone diminui fogo
        Q = q.nextInt(4) + 1; // máximo 4 itens
        for (int i = 0; i < Q; i++) {
            L = l.nextInt(9) + 1;
            C = c.nextInt(15) + 1;
            if (cenario[L][C] == 1) { // verifica se é tijolo
                cenario[L][C] = 15;
            } else {
                i--;
            }
        }
        // disposição do icone aumenta fogo
        Q = q.nextInt(4) + 1; // máximo 4 itens
        for (int i = 0; i < Q; i++) {
            L = l.nextInt(9) + 1;
            C = c.nextInt(15) + 1;
            if (cenario[L][C] == 1) {// verifica se é tijolo
                cenario[L][C] = 16;
            } else {
                i--;
            }
        }
    }
    // os métodos abaixo, servem para alterar as teclas com o jogo rodando
    public void setAtirar(int atirar) {
        this.atirar = atirar;
    }

    public void setBaixo(int baixo) {
        this.baixo = baixo;
    }

    public void setCima(int cima) {
        this.cima = cima;
    }

    public void setDireita(int direita) {
        this.direita = direita;
    }

    public void setEsquerda(int esquerda) {
        this.esquerda = esquerda;
    }
}
